#!/usr/bin/python
# -*- coding: utf-8 -*-

#bumping balls in 3D
#Copyright (C) 2009  Janosch Maier <janosch_maier@online.de>
#For further information see <http://phylu.bplaced.de>

#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.


import sys # Systemfunktionen und Konstanten (z.B. sys.exit)
import time # Zeitfunktionen
import math # Mathematische Funktionen
from OpenGL.GL import * # Import von OpenGL und The OpenGL Utillity Toolkit
from OpenGL.GLE import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from entry import * # Ball Klasse einbinden
from ball import * # Umgebungsklasse einbinden

## Umgebungsklasse definieren ##
class enviroment: 
    
    def __init__(self, width, height): # Initialfunktion definieren
        ## Variablen uebergeben ##
        self.width = width # Fensterbreite
        self.height = height # Fensterhöhe
        self.zoom_start = -40.0
        self.zoom = self.zoom_start # Zoomfaktor
        self.lastx = 0.0 # Maus X-Drehungswert
        self.lasty = 0.0 # Maus Y-Drehungswert
        self.counterBalls = 0 # Kugelzähler definieren
        self.sphere = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9] # Kugeln vorbelgen
        self.cubeLength = 20.0
        self.start = False
        self.gravityOn = False
        if sys.platform == "linux2": # Betriebssystem erkennen
            self.divisor = 1000 # Faktoren, um die Geschwindigkeiten auf dem Bildschirm darstellbar zu machen
            self.gravity = 9.81/1000 # Weiterer Faktor, damit die Beschleunigung pro Sekunde bei 9,81 liegt, und nich pro Rechenschritt
        else: # Andere Faktoren, da Darstellungsgeschwindigkeit unter Windows deutlich langsamer ist
            self.divisor = 100
            self.gravity = 9.81/50
        self.makeBall(self.divisor)
        self.makeBall(self.divisor)
        ## -------------------- ##
    
    
    ## Pause Funktion ##
    def pause(self):
        if self.start == False:
            self.start = True
        else:
            self.start  = False
    ## -------------- ##
    
    ## Schwerkraft Funktion ##
    def switchGravity(self):
        if self.gravityOn == False:
            self.gravityOn = True
        else:
            self.gravityOn  = False
    ## -------------- ##
    
    ## Bearbeiten Fenster aufrufen ##
    def editParams(self, i):
        var = self.sphere[i].getVars()
        self.window = Application(var)
        self.window.master.title("Kugel " + str(i) + " Bearbeiten")
        self.window.mainloop()
        var = self.window.getVars()
        self.sphere[i].setVars(var)
    
    
    ## Kugeln ##
    def makeBall(self, divisor, r=0.0, m=0.0, rx=0.0, ry=0.0, rz=0.0, vx=0.0, vy=0.0, vz=0.0, red=0.0, green=0.0, blue=0.0):
        self.counterBalls += 1 # Kugelzähler erhöhen
        self.sphere[self.counterBalls] = ball(r, m, rx, ry, rz, vx, vy, vz, red, green, blue, divisor) # Klasse Ball aufrufen
        
    def destroyBalls(self):
        i = 1
        while i <= self.counterBalls:
            self.sphere[i].setVars([0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])
            i += 1


    ## Berechnungen ##
    def calculate(self):
        ## Stoß mit der Wand ##
        i = 1
        while i <= self.counterBalls:
            if (self.sphere[i].rx - self.sphere[i].r <= - self.cubeLength / 2 and self.sphere[i].vx < 0) or (self.sphere[i].rx + self.sphere[i].r >= self.cubeLength / 2 and self.sphere[i].vx > 0): # Stoß an der X-Seite des Würfels
                self.sphere[i].vx = - self.sphere[i].vx # vx umdrehen
            if (self.sphere[i].ry - self.sphere[i].r <= - self.cubeLength / 2 and self.sphere[i].vy < 0) or (self.sphere[i].ry + self.sphere[i].r >= self.cubeLength / 2 and self.sphere[i].vy > 0): # Stoß an der Y-Seite des Würfels
                self.sphere[i].vy = - self.sphere[i].vy # vy umdrehen
            if (self.sphere[i].rz - self.sphere[i].r <= - self.cubeLength / 2 and self.sphere[i].vz < 0) or (self.sphere[i].rz + self.sphere[i].r >= self.cubeLength / 2 and self.sphere[i].vz > 0): # Stoß an der X-Seite des Würfels
                self.sphere[i].vz = - self.sphere[i].vz # vz umdrehen
            i += 1
        # ---------------- ##

        ## Stoß mit anderen Kugeln ##
        i = 1
        while i <= self.counterBalls:
            j = i + 1
            while j <= self.counterBalls:
                distance = math.sqrt ( math.pow((self.sphere[i].rx - self.sphere[j].rx), 2) + math.pow((self.sphere[i].ry - self.sphere[j].ry), 2) + math.pow((self.sphere[i].rz - self.sphere[j].rz), 2) ) # Entfernung der beiden Kugelmittelpunkte von kugel i und kugel j berechnen
                if distance <= self.sphere[i].r + self.sphere[j].r: # Es passiert ein Stoß!
                
                    #Vorzeichenüberlegung                    
                    if (self.sphere[i].rx < self.sphere[j].rx and self.sphere[i].vx > self.sphere[j].vx) or (self.sphere[i].rx > self.sphere[j].rx and self.sphere[i].vx < self.sphere[j].vx) or (self.sphere[i].ry < self.sphere[j].ry and self.sphere[i].vy > self.sphere[j].vy) or  (self.sphere[i].ry > self.sphere[j].ry and self.sphere[i].vy < self.sphere[j].vy) or (self.sphere[i].rz < self.sphere[j].rz and self.sphere[i].vz > self.sphere[j].vz) or (self.sphere[i].rz > self.sphere[j].rz and self.sphere[i].vz < self.sphere[j].vz): # Die Kugeln bewegen sich aufeinander zu                    
 
                        # Richtungsvektor der Normalen
                        nrx = self.sphere[j].rx - self.sphere[i].rx
                        nry = self.sphere[j].ry - self.sphere[i].ry
                        nrz = self.sphere[j].rz - self.sphere[i].rz

                        # Kugel i
                        i_vnx = (nrx * (  nrx * self.sphere[i].vx + nry * self.sphere[i].vy + nrz * self.sphere[i].vz ) ) / ( math.pow(nrx, 2) + math.pow(nry, 2)+ math.pow(nrz, 2) )
                        i_vny = (nry * (  nrx * self.sphere[i].vx + nry * self.sphere[i].vy + nrz * self.sphere[i].vz ) ) / ( math.pow(nrx, 2) + math.pow(nry, 2)+ math.pow(nrz, 2) )
                        i_vnz = (nrz * (  nrx * self.sphere[i].vx + nry * self.sphere[i].vy + nrz * self.sphere[i].vz ) ) / ( math.pow(nrx, 2) + math.pow(nry, 2)+ math.pow(nrz, 2) )
                        i_vex = self.sphere[i].vx - i_vnx
                        i_vey = self.sphere[i].vy - i_vny
                        i_vez = self.sphere[i].vz - i_vnz
                        
                        # Kugel j
                        j_vnx = (nrx * (  nrx * self.sphere[j].vx + nry * self.sphere[j].vy + nrz * self.sphere[j].vz ) ) / ( math.pow(nrx, 2) + math.pow(nry, 2)+ math.pow(nrz, 2) )
                        j_vny = (nry * (  nrx * self.sphere[j].vx + nry * self.sphere[j].vy + nrz * self.sphere[j].vz ) ) / ( math.pow(nrx, 2) + math.pow(nry, 2)+ math.pow(nrz, 2) )
                        j_vnz = (nrz * (  nrx * self.sphere[j].vx + nry * self.sphere[j].vy + nrz * self.sphere[j].vz ) ) / ( math.pow(nrx, 2) + math.pow(nry, 2)+ math.pow(nrz, 2) )
                        j_vex = self.sphere[j].vx - j_vnx
                        j_vey = self.sphere[j].vy - j_vny
                        j_vez = self.sphere[j].vz - j_vnz
                        
                        # Stossberechnung
                        i_unx = (self.sphere[i].m * i_vnx + self.sphere[j].m * (2 * j_vnx - i_vnx) ) / ( self.sphere[i].m + self.sphere[j].m )
                        i_uny = (self.sphere[i].m * i_vny + self.sphere[j].m * (2 * j_vny - i_vny) ) / ( self.sphere[i].m + self.sphere[j].m )
                        i_unz = (self.sphere[i].m * i_vnz + self.sphere[j].m * (2 * j_vnz - i_vnz) ) / ( self.sphere[i].m + self.sphere[j].m )
                        
                        j_unx = (self.sphere[j].m * j_vnx + self.sphere[i].m * (2 * i_vnx - j_vnx) ) / ( self.sphere[i].m + self.sphere[j].m )
                        j_uny = (self.sphere[j].m * j_vny + self.sphere[i].m * (2 * i_vny - j_vny) ) / ( self.sphere[i].m + self.sphere[j].m )
                        j_unz = (self.sphere[j].m * j_vnz + self.sphere[i].m * (2 * i_vnz - j_vnz) ) / ( self.sphere[i].m + self.sphere[j].m )
                                
                        # Geschwindigkeiten setzen
                        self.sphere[i].setSpeed((i_vex + i_unx), (i_vey + i_uny), (i_vez + i_unz))
                        self.sphere[j].setSpeed((j_vex + j_unx), (j_vey + j_uny), (j_vez + j_unz))
                        
                j += 1
            i += 1
            
        # Evtl. die Beschleunigung durh die Schwerkraft setzen
        if self.gravityOn == True:
            i = 1
            while i <= self.counterBalls:
                 self.sphere[i].setSpeed(self.sphere[i].vx, self.sphere[i].vy - self.gravity/self.divisor, self.sphere[i].vz)
                 i += 1
        ## --------------- ##

        ## Geschwindigkeit -> Ort ##
        i = 1
        while i <= self.counterBalls: # Für jede Kugel:
            x = self.sphere[i].rx + self.sphere[i].vx
            y = self.sphere[i].ry + self.sphere[i].vy
            z = self.sphere[i].rz + self.sphere[i].vz
            self.sphere[i].setPosition(x, y, z)
            i += 1
        # ---------------------- ##
    ## ------------ ##



    ## Zeichnen ##
    def drawScene(self):
    
        if self.start == True:
            self.calculate() # Berechnung, für das nächste mal zeichnen

        glClearColor(0.0, 0.0, 0.0, 0.0) # Farbe fuer leeres Fenster setzen
        glClearDepth(1.0); # Tiefenbufferleerung definieren
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT) # Farb- und Tiefenbuffer leeren
        
        glMatrixMode(GL_MODELVIEW)
        
        i = 1
        while i <= self.counterBalls: # Für jede Kugel:
            glLoadIdentity()
            glColor3f(self.sphere[i].red, self.sphere[i].green, self.sphere[i].blue) # Farbe Kugel
            glTranslatef(self.sphere[i].rx, self.sphere[i].ry, self.sphere[i].rz) # Ort Kugel
            self.whatever = gluNewQuadric() # Kugel1 definieren
            gluSphere(self.whatever, self.sphere[i].r, 50, 50) # Kugel 1 zeichnen
            i += 1

        glLoadIdentity()
        glColor4f(0.5, 0.5, 1.0, 0.5) # Farbe Kiste
        glutSolidCube(self.cubeLength) # Kiste bauen


        
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity() # Projektionsmatrix loeschen
        gluPerspective(45.0, float(self.height)/float(self.width), 0.01, 500.0) # Perspektivische Sicht einschalten
        glTranslatef(0.0, 0.0, self.zoom) # Ansicht verschieben
        glRotatef(self.lastx, 0.0, 1.0, 0.0) # In X-Richtung drehen
        glRotatef(self.lasty, 1.0, 0.0, 0.0) # In Y-Richtung drehen
       
        
        #glFlush() # Zeichnen erzwingen -> (falls Programm ueber Netzwerk laeuft)
        glutSwapBuffers() # Zeichenbuffer wechseln -> fluessigeres Zeichnen glFlush() ist mit inbegriffen
    # ---------- #
    
    def drawIdleScene(self):
        #time.sleep(0.04)
        self.drawScene()
    
    def resizeScene(self, width, height):
        if height == 0: # Teilen durch null verhindern
           height = 1
        if width <= height: # Anzeige quadratisch machen mit der Seitenlaenge der kleineren Seite
            self.width = width
            self.height = width
        elif height < width:
            self.height = height
            self.width = height
        xpos = (width - self.width) / 2 # Anzeige im Fenster zentrieren
        ypos = (height - self.height) / 2
        glViewport (xpos, ypos, self.width, self.height) # Anzeige setzen
        glutPostRedisplay() # Fenster als als "muss aktualisiert werden" markieren

    ## Tastatur ##
    def keyPressed(self, key, mousex, mousey): # Taste gedrueckt
        if key == "\33": # Escape gedrueckt \33 == Esc ASCII-Code
            sys.exit() # Programm beenden
        elif key == "+": # Taste + gedrueckt
            self.zoom = self.zoom + 0.1 # reinzoomen
        elif key == "-": # Taste - gedrueckt
            self.zoom = self.zoom - 0.1 # rauszoomen
        elif key == "1": # Auf Anfangsposition setzen
            self.editParams(1)
        elif key == "2": # Sicht auf Anfan setzen + herauszoomen
            self.editParams(2)
        elif key == "s":
            self.zoom = self.zoom_start
            self.lastx = 0
            self.lasty = 0
        elif key == "d":
            self.destroyBalls()
        elif key == "g":
            self.switchGravity()
        elif key == "\40": # Leertaste
            self.pause()
        glutPostRedisplay() # Fenster als als "muss aktualisiert werden" markieren

    def keyReleased(self, key, mousex, mousey): # Taste losgelassen
        pass # Nichts passiert
    ## ---------##
    
    ## Maus ##
    def keyMouse(self, button, state, x, y): # Maustaste druecken oder loslassen
        self.mouseButton = button # Maustaste uebergeben
        if button == 0: # Linke Maustaste
            if state == 0: # druecken
                self.xstart = x - self.lastx # Wert der Mauskoordinaten beim druecken merken
                self.ystart = y - self.lasty
        elif button == 2: # Rechte Maustaste
            if state == 0:
                self.zstart = y - self.zoom * 10 # * 10, um keyMouseMotion / 10 auszugleichen
        glutPostRedisplay()

    def keyMouseMotion(self, x, y): # Mauskoordinaten, waehrend eine Taste gedrueckt ist
        if self.mouseButton == 0:
            self.lastx = x - self.xstart # Beweungsrichtung der Maus registrieren
            self.lasty = y - self.ystart
        if self.mouseButton == 2:
            self.zoom = (y - self.zstart) / 10 # /10 um Bewegung weniger sensibel zu machen
        glutPostRedisplay()
    ## ---- ##
