#!/usr/bin/python
# -*- coding: utf-8 -*-

#bumping balls in 3D
#Copyright (C) 2009  Janosch Maier <janosch_maier@online.de>
#For further information see <http://phylu.bplaced.de>

#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#For a copy of the GNU General Public License see <http://www.gnu.org/licenses/>.

import sys # Systemfunktionen und Konstanten (z.B. sys.exit)
import time # Zeitfunktionen
import math # Mathematische Funktionen
from OpenGL.GL import * # Import von OpenGL und The OpenGL Utillity Toolkit
from OpenGL.GLE import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
from enviroment import * # Umgebungsklasse einbinden
import os # Informationen über das Betriebssystem

## Klasse des Hauptfensters ##
class InitGL: 
    
    def __init__(self, x, y, width, height, name): # Klasse initialisieren
    
        glutInit(sys.argv) # GLUT Initialisieren
        self.surroundings = enviroment(width, height) # Umgebungsklasse aufrufen
        
        ## Glut Modus ##
        glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH) # Double Buffer einschalten -> weicheres zeichnen, rgba-Farbmodus einschalten, Tiefenbuffer aktivieren
        glutInitWindowSize(width, height) # Fenstergroesse setzen
        glutInitWindowPosition(x, y) # Fensterposition setzen
        glutCreateWindow(name) # Fenstername setzen
        ## --------- ##

        glMaterial(GL_FRONT, GL_SPECULAR, (1.0, 1.0, 1.0, 1.0))
        glMaterial(GL_FRONT, GL_SHININESS, 50.0)
        glEnable(GL_COLOR_MATERIAL)
        glLight(GL_LIGHT0, GL_POSITION, (1.0, 1.0, 1.0, 0.0))

        glEnable(GL_LIGHTING) # Beleuchtung aktivieren
        glEnable(GL_LIGHT0) # Lichtquelle 1 setzen
        #glDepthFunc(GL_LEQUAL)
        
        glEnable(GL_DEPTH_TEST) # Tiefenbuffervergleich aktivieren
        
        glEnable(GL_BLEND) # Blending aktivieren (Alpha-Kanal wird wirksam)
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) # Blending Parameter setzen
        
        
        ## Allgemeine Funktionen
        glutIdleFunc(self.surroundings.drawIdleScene) # Festlegen, welche Funktion aufgerufen wird, wenn keine Eingabe erfolgt
        glutDisplayFunc(self.surroundings.drawScene) # Funktion, die aufgerufen wird, wenn das Display aktualisiert werden muss
        glutReshapeFunc(self.surroundings.resizeScene) # Groesse Veraendern
        
        ## Tastatur
        glutKeyboardFunc(self.surroundings.keyPressed) # Funktion, wenn eine Taste gedrueckt wird
        glutKeyboardUpFunc(self.surroundings.keyReleased) # Taste loslassen
        
        ## Maus
        glutMouseFunc(self.surroundings.keyMouse) # Maustasten druecken und loslassen
        glutMotionFunc(self.surroundings.keyMouseMotion) # Bewgend der Maus be gedrueckten Tasten


if __name__ == '__main__': # Wenn die Datei das Hauptfenster ist, und sie nicht importiert wird
    program = InitGL(0, 0, 700, 700, "Simulation von elastischen Stoesen vieler Kugeln im 3-dimensionalen Raum") # Fensternamen setztn
    
    #Kugel Startparameter: [radius, masse, ortx, orty, ortz, geschwindigkeitx, geschwindigkeity, geschwindigkeitz, rot, gruen, blau]
    program.surroundings.makeBall(1000)
    program.surroundings.makeBall(1000)
    program.surroundings.makeBall(1000)
    program.surroundings.makeBall(1000)
    program.surroundings.sphere[1].setVars([3.0, 3.0, 5.0, 1.0, 0.0, -1.0, 0.0, 2.0, 1.0, 0.0, 1.0])
    program.surroundings.sphere[2].setVars([2.0, 2.0, 0.0, -5.0, 3.0, 0.0, -3.0, 0.0, 0.0, 1.0, 1.0])
    program.surroundings.sphere[3].setVars([1.0, 1.0, -2.0, 8.0, 5.0, 12.0, -3.0, 0.0, 1.0, 1.0, 0.0])
    program.surroundings.sphere[4].setVars([0.5, 0.5, 2.0, 1.0, -5.0, 0.0, -3.0, 0.0, 0.0, 0.0, 1.0])
    program.surroundings.sphere[5].setVars([3.0, 3.0, -5.0, -2.0, 0.0, 0.0, 0.0, 15.0, 0.0, 1.0, 0.0])
    program.surroundings.sphere[6].setVars([2.5, 8.5, 1.0, 3.5, -7.0, -20.0, -3.0, -10.0, 1.0, 0.0, 0.0])

    glutMainLoop() # Hauptschleife -> Idle + Auf Eingabe warten
